package Kakuro.parser;

import Kakuro.KakuroProblem;
import Kakuro.Cell;
import Kakuro.SumAlldiffPb;
import Kakuro.Sequence;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created by IntelliJ IDEA.
 * User: hcambaza
 * Date: 10 janv. 2008
 * Time: 22:23:25
 * To change this template use File | Settings | File Templates.
 */
public class Reader {
	
	private BufferedReader in;
	
	public Reader(String file) throws FileNotFoundException {
		in = new BufferedReader(new FileReader(file));
	}

	/**
	 * Parse the file and create the data of the kakuro problem, namely :
	 * - the matrix of cells
	 * - the matrix of SumAlldiffPb
	 * - the list of sequences
	 * @throws IOException 
	 * @throws NumberFormatException 
	 */
	public KakuroProblem loadNext() throws IOException {
		//data(mix,301,easy,12,12,
		Pattern startPattern = Pattern.compile("data\\((.*?),(\\d*?),(.*?),(\\d*?),(\\d*?),.*?");
		//[](x/x  ,x/x  ,7/7  ,x    ,x    ,x    ,10/x ,x/x  ,7/x  ,3/x  ,15/x ,x/x  ),
		Pattern rowPattern = Pattern.compile("\\s*\\[\\]\\((.*?)\\),?\\s*?");
		
		
		
		Pattern endPattern = Pattern.compile("\\s*\\)\\)\\.\\s*");
	
		String str, name;
		int nblig, nbcol, pbNbr;
		while((str = in.readLine()) != null){
			if(str.startsWith("/*")){
				do str = in.readLine();
				while(!str.startsWith("*/"));
			}
			
			Matcher m = startPattern.matcher(str);
			if(m.matches()){
				name = m.group(1)+" "+m.group(5);
				try{ pbNbr = new Integer(m.group(2)); } catch( NumberFormatException e) {pbNbr = 0;}
				nblig = new Integer(m.group(4));
				nbcol = new Integer(m.group(5));
				System.out.println(pbNbr+" "+name);
				
				int rowNb = 0;
				String[][] inst = new String[nblig][nbcol];
				
				str = in.readLine();
				while(!endPattern.matcher(str).matches()){
					Matcher row = rowPattern.matcher(str);
					if(row.matches()){
						try{
						String[] cells = row.group(1).split(",");
						for(int i = 0; i < cells.length; i++)
							inst[rowNb][i] = cells[i].trim();
						rowNb++;
						}catch(ArrayIndexOutOfBoundsException e){System.out.println(row.group()); e.printStackTrace(); System.exit(0); }
					}
					str = in.readLine();
				}
				
				/*for(int i = 0; i < inst.length; i++){
					System.out.print("\t");
					for(int j = 0; j < inst[0].length; j++)
						System.out.print("["+inst[i][j]+"]");
					System.out.println();
				}*/
				
				KakuroProblem kprob = new KakuroProblem(name, pbNbr);
				kprob.createMatrix(nblig, nbcol);
				
				for(int i = 0; i < inst.length; i++)
					for(int j = 0; j < inst[0].length; j++)
						if(inst[i][j].length() == 1)
							kprob.setCell(new Cell(i, j));
				
				String[] sum;
				for(int i = 0; i < inst.length; i++)
					for(int j = 0; j < inst[0].length; j++)
						if(inst[i][j].length() != 1){
							sum = inst[i][j].split("/");
							if(!sum[0].equals("x")){
								int pos = i;
								
								do pos++;
								while(pos < inst.length && inst[pos][j].length() == 1);
								pos--;
								
								SumAlldiffPb sumAD = kprob.setSumAlldiffPb(pos - i, new Integer(sum[0]));
								
								Sequence seq = new Sequence(kprob.getCell(i+1, j), kprob.getCell(pos, j), sumAD);
								kprob.addSequence(seq);
								for(int a = i+1; a <= pos; a++)
									kprob.getCell(a, j).setVerticalSeq(seq);
							}
							if(!sum[1].equals("x")){
								int pos = j;
								
								do pos++;
								while(pos < inst[0].length && inst[i][pos].length() == 1);
								pos--;
								
								SumAlldiffPb sumAD = kprob.setSumAlldiffPb(pos - j, new Integer(sum[1]));
								
								Sequence seq = new Sequence(kprob.getCell(i, j+1), kprob.getCell(i, pos), sumAD);
								kprob.addSequence(seq);
								for(int a = j+1; a <= pos; a++)
									kprob.getCell(i, a).setHorizontalSeq(seq);
							}
						}
				return kprob;
			}
		}
		
		in.close();
		return null;
		/*
		KakuroProblem kprob = new KakuroProblem();
		int nblig = 0; //read the nb lig
		int nbcol = 0; //read the nb col
		kprob.createMatrix(nblig,nbcol);
		//...
		 */
	}

	public static void main(String[] args) {
		try {
			Reader temp = new Reader("data.ecl");
			while(temp.loadNext()!= null);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}


}
